@@ -61,6 +61,8 @@
 
 struct _WnckPagerPrivate
 {
+  WnckHandle *handle;
+
   WnckScreen *screen;
 
   int n_rows; /* really columns for vertical orientation */
@@ -89,6 +91,17 @@ struct _WnckPagerPrivate
   guint dnd_time; /* time of last event during dnd (for delayed workspace activation) */
 };
 
+enum
+{
+  PROP_0,
+
+  PROP_HANDLE,
+
+  LAST_PROP
+};
+
+static GParamSpec *pager_properties[LAST_PROP] = { NULL };
+
 G_DEFINE_TYPE_WITH_PRIVATE (WnckPager, wnck_pager, GTK_TYPE_WIDGET);
 
 enum
@@ -122,8 +135,6 @@ static void     wnck_pager_get_preferred_height_for_width (GtkWidget *widget,
                                                            int        width,
                                                            int       *minimum_height,
                                                            int       *natural_height);
-static void     wnck_pager_size_allocate (GtkWidget        *widget,
-                                          GtkAllocation    *allocation);
 static gboolean wnck_pager_draw          (GtkWidget        *widget,
                                           cairo_t          *cr);
 static gboolean wnck_pager_button_press  (GtkWidget        *widget,
@@ -234,6 +245,67 @@ wnck_pager_init (WnckPager *pager)
   gtk_widget_set_can_focus (GTK_WIDGET (pager), TRUE);
 }
 
+static void
+wnck_pager_get_property (GObject    *object,
+                         guint       property_id,
+                         GValue     *value,
+                         GParamSpec *pspec)
+{
+  WnckPager *self;
+
+  self = WNCK_PAGER (object);
+
+  switch (property_id)
+    {
+      case PROP_HANDLE:
+        g_value_set_object (value, self->priv->handle);
+        break;
+
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
+static void
+wnck_pager_set_property (GObject      *object,
+                         guint         property_id,
+                         const GValue *value,
+                         GParamSpec   *pspec)
+{
+  WnckPager *self;
+
+  self = WNCK_PAGER (object);
+
+  switch (property_id)
+    {
+      case PROP_HANDLE:
+        g_assert (self->priv->handle == NULL);
+        self->priv->handle = g_value_dup_object (value);
+        break;
+
+      default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+        break;
+    }
+}
+
+static void
+install_properties (GObjectClass *object_class)
+{
+  pager_properties[PROP_HANDLE] =
+    g_param_spec_object ("handle",
+                         "handle",
+                         "handle",
+                         WNCK_TYPE_HANDLE,
+                         G_PARAM_CONSTRUCT_ONLY |
+                         G_PARAM_READWRITE |
+                         G_PARAM_EXPLICIT_NOTIFY |
+                         G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, LAST_PROP, pager_properties);
+}
+
 static void
 wnck_pager_class_init (WnckPagerClass *klass)
 {
@@ -241,6 +313,8 @@ wnck_pager_class_init (WnckPagerClass *klass)
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->finalize = wnck_pager_finalize;
+  object_class->get_property = wnck_pager_get_property;
+  object_class->set_property = wnck_pager_set_property;
 
   widget_class->realize = wnck_pager_realize;
   widget_class->unrealize = wnck_pager_unrealize;
@@ -249,7 +323,6 @@ wnck_pager_class_init (WnckPagerClass *klass)
   widget_class->get_preferred_width_for_height = wnck_pager_get_preferred_width_for_height;
   widget_class->get_preferred_height = wnck_pager_get_preferred_height;
   widget_class->get_preferred_height_for_width = wnck_pager_get_preferred_height_for_width;
-  widget_class->size_allocate = wnck_pager_size_allocate;
   widget_class->draw = wnck_pager_draw;
   widget_class->button_press_event = wnck_pager_button_press;
   widget_class->button_release_event = wnck_pager_button_release;
@@ -265,6 +338,8 @@ wnck_pager_class_init (WnckPagerClass *klass)
   widget_class->drag_end = wnck_pager_drag_end;
   widget_class->query_tooltip = wnck_pager_query_tooltip;
 
+  install_properties (object_class);
+
   gtk_widget_class_set_css_name (widget_class, "wnck-pager");
 }
 
@@ -287,6 +362,8 @@ wnck_pager_finalize (GObject *object)
       pager->priv->dnd_activate = 0;
     }
 
+  g_clear_object (&pager->priv->handle);
+
   G_OBJECT_CLASS (wnck_pager_parent_class)->finalize (object);
 }
 
@@ -294,12 +371,16 @@ static void
 _wnck_pager_set_screen (WnckPager *pager)
 {
   GdkScreen *gdkscreen;
+  int screen_number;
 
   if (!gtk_widget_has_screen (GTK_WIDGET (pager)))
     return;
 
   gdkscreen = gtk_widget_get_screen (GTK_WIDGET (pager));
-  pager->priv->screen = wnck_screen_get (gdk_x11_screen_get_screen_number (gdkscreen));
+  screen_number = gdk_x11_screen_get_screen_number (gdkscreen);
+
+  pager->priv->screen = wnck_handle_get_screen (pager->priv->handle,
+                                                screen_number);
 
   if (!wnck_pager_set_layout_hint (pager))
     {
@@ -687,62 +768,6 @@ wnck_pager_get_preferred_height_for_width (GtkWidget *widget,
   *natural_height = *minimum_height = MAX (height, 0);
 }
 
-static gboolean
-_wnck_pager_queue_resize (gpointer data)
-{
-  gtk_widget_queue_resize (GTK_WIDGET (data));
-  return FALSE;
-}
-
-static void
-wnck_pager_size_allocate (GtkWidget      *widget,
-                          GtkAllocation  *allocation)
-{
-  WnckPager *pager;
-  int workspace_size;
-  GtkBorder padding;
-  int width;
-  int height;
-
-  pager = WNCK_PAGER (widget);
-
-  width = allocation->width;
-  height = allocation->height;
-
-  _wnck_pager_get_padding (pager, &padding);
-  width  -= padding.left + padding.right;
-  height -= padding.top + padding.bottom;
-
-  g_assert (pager->priv->n_rows > 0);
-
-  if (pager->priv->orientation == GTK_ORIENTATION_VERTICAL)
-    {
-      if (pager->priv->show_all_workspaces)
-	workspace_size = (width - (pager->priv->n_rows - 1))  / pager->priv->n_rows;
-      else
-	workspace_size = width;
-    }
-  else
-    {
-      if (pager->priv->show_all_workspaces)
-	workspace_size = (height - (pager->priv->n_rows - 1))/ pager->priv->n_rows;
-      else
-	workspace_size = height;
-    }
-
-  workspace_size = MAX (workspace_size, 1);
-
-  if (workspace_size != pager->priv->workspace_size)
-    {
-      pager->priv->workspace_size = workspace_size;
-      g_idle_add (_wnck_pager_queue_resize, pager);
-      return;
-    }
-
-  GTK_WIDGET_CLASS (wnck_pager_parent_class)->size_allocate (widget,
-                                                             allocation);
-}
-
 static void
 get_workspace_rect (WnckPager    *pager,
                     int           space,
@@ -2252,11 +2277,34 @@ wnck_pager_new (void)
 {
   WnckPager *pager;
 
-  pager = g_object_new (WNCK_TYPE_PAGER, NULL);
+  pager = g_object_new (WNCK_TYPE_PAGER,
+                        "handle", _wnck_get_handle (),
+                        NULL);
 
   return GTK_WIDGET (pager);
 }
 
+/**
+ * wnck_pager_new_with_handle:
+ * @handle: a #WnckHandle
+ *
+ * Creates a new #WnckPager. The #WnckPager will show the #WnckWorkspace of the
+ * #WnckScreen it is on.
+ *
+ * Returns: a newly created #WnckPager.
+ */
+GtkWidget *
+wnck_pager_new_with_handle (WnckHandle *handle)
+{
+  WnckPager *self;
+
+  self = g_object_new (WNCK_TYPE_PAGER,
+                       "handle", handle,
+                       NULL);
+
+  return GTK_WIDGET (self);
+}
+
 static gboolean
 wnck_pager_set_layout_hint (WnckPager *pager)
 {
