@@ -291,7 +291,7 @@ static GOptionEntry space_entries[] = {
 	{ NULL }
 };
 
-static void clean_up (void);
+static void clean_up (WnckHandle *handle);
 
 /* this part is mostly stolen from xutils.c */
 typedef struct
@@ -481,12 +481,12 @@ option_parse (const char  *option_name,
 static gboolean
 set_mode (int         new_mode,
           const char *option,
-          gboolean    list)
+          gboolean    use_list)
 {
   switch (mode)
     {
       case INVALID_MODE:
-        if (list)
+        if (use_list)
           g_assert_not_reached ();
 
         mode = new_mode;
@@ -494,7 +494,7 @@ set_mode (int         new_mode,
       case SCREEN_READ_MODE:
         if (new_mode == SCREEN_READ_MODE || new_mode == SCREEN_WRITE_MODE)
           mode = new_mode;
-        else if (list)
+        else if (use_list)
           mode = SCREEN_LIST_MODE;
         else
           {
@@ -526,7 +526,7 @@ set_mode (int         new_mode,
       case WORKSPACE_READ_MODE:
         if (new_mode == WORKSPACE_READ_MODE || new_mode == WORKSPACE_WRITE_MODE)
           mode = new_mode;
-        else if (list)
+        else if (use_list)
           mode = WORKSPACE_LIST_MODE;
         else
           {
@@ -558,7 +558,7 @@ set_mode (int         new_mode,
           }
         break;
       case APPLICATION_READ_MODE:
-        if (list)
+        if (use_list)
           mode = APPLICATION_LIST_MODE;
         else if (new_mode != APPLICATION_READ_MODE)
           {
@@ -580,7 +580,7 @@ set_mode (int         new_mode,
           }
         break;
       case CLASS_GROUP_READ_MODE:
-        if (list)
+        if (use_list)
           mode = CLASS_GROUP_LIST_MODE;
         else if (new_mode != CLASS_GROUP_READ_MODE)
           {
@@ -1673,6 +1673,7 @@ static gboolean
 wm_state_set (Display *display,
               Window   window)
 {
+  GdkDisplay *gdk_display;
   Atom    wm_state;
   gulong  nitems;
   gulong  bytes_after;
@@ -1681,16 +1682,19 @@ wm_state_set (Display *display,
   int     ret_format;
   int     err, result;
 
+  gdk_display = gdk_x11_lookup_xdisplay (display);
+  g_assert (gdk_display != NULL);
+
   wm_state = gdk_x11_get_xatom_by_name ("WM_STATE");
 
-  gdk_error_trap_push ();
+  gdk_x11_display_error_trap_push (gdk_display);
   result = XGetWindowProperty (display,
                                window,
                                wm_state,
                                0, G_MAXLONG,
                                False, wm_state, &ret_type, &ret_format, &nitems,
                                &bytes_after, (gpointer) &prop);
-  err = gdk_error_trap_pop ();
+  err = gdk_x11_display_error_trap_pop (gdk_display);
   if (err != Success ||
       result != Success)
     return FALSE;
@@ -1704,22 +1708,27 @@ wm_state_set (Display *display,
 }
 
 static WnckWindow *
-find_managed_window (Display *display,
-                     Window   window)
+find_managed_window (WnckHandle *handle,
+                     Display    *display,
+                     Window      window)
 {
+  GdkDisplay *gdk_display;
   Window      root;
   Window      parent;
   Window     *kids = NULL;
   WnckWindow *retval;
-  guint       nkids;
-  int         i, result;
+  guint       i, nkids;
+  int         result;
 
   if (wm_state_set (display, window))
-    return wnck_window_get (window);
+    return wnck_handle_get_window (handle, window);
 
-  gdk_error_trap_push ();
+  gdk_display = gdk_x11_lookup_xdisplay (display);
+  g_assert (gdk_display != NULL);
+
+  gdk_x11_display_error_trap_push (gdk_display);
   result = XQueryTree (display, window, &root, &parent, &kids, &nkids);
-  if (gdk_error_trap_pop () || !result)
+  if (gdk_x11_display_error_trap_pop (gdk_display) || !result)
     return NULL;
 
   retval = NULL;
@@ -1728,11 +1737,11 @@ find_managed_window (Display *display,
     {
       if (wm_state_set (display, kids [i]))
         {
-          retval = wnck_window_get (kids [i]);
+          retval = wnck_handle_get_window (handle, kids [i]);
           break;
         }
 
-      retval = find_managed_window (display, kids [i]);
+      retval = find_managed_window (handle, display, kids [i]);
       if (retval != NULL)
         break;
     }
@@ -1744,19 +1753,22 @@ find_managed_window (Display *display,
 }
 
 static void
-handle_button_press_event (Display *dpy, XIDeviceEvent *event)
+handle_button_press_event (WnckHandle    *handle,
+                           Display       *dpy,
+                           XIDeviceEvent *event)
 {
   if (event->child == None)
     return;
 
-  got_from_user = find_managed_window (dpy, event->child);
+  got_from_user = find_managed_window (handle, dpy, event->child);
 }
 
 static GdkFilterReturn
 target_filter (GdkXEvent *gdk_xevent,
-               GdkEvent  *event,
+               GdkEvent  *gdk_event,
                gpointer   data)
 {
+  WnckHandle *handle = data;
   XEvent *xevent = (XEvent *) gdk_xevent;
   XGenericEventCookie *cookie = &xevent->xcookie;
 
@@ -1771,13 +1783,13 @@ target_filter (GdkXEvent *gdk_xevent,
       switch (event->evtype)
         {
           case XI_ButtonPress:
-            handle_button_press_event (cookie->display, event);
-            clean_up ();
+            handle_button_press_event (handle, cookie->display, event);
+            clean_up (handle);
             return GDK_FILTER_REMOVE;
           case XI_KeyPress:
             if (event->detail == XKeysymToKeycode (cookie->display, XK_Escape))
               {
-                clean_up ();
+                clean_up (handle);
                 return GDK_FILTER_REMOVE;
               }
             break;
@@ -1789,93 +1801,63 @@ target_filter (GdkXEvent *gdk_xevent,
   return GDK_FILTER_CONTINUE;
 }
 
+static void
+prepare (GdkSeat   *seat,
+         GdkWindow *window,
+         gpointer   user_data)
+{
+  gdk_window_show_unraised (window);
+}
+
 static gboolean
 get_target (gpointer data)
 {
-  GdkGrabStatus     status;
-  GdkDeviceManager *dev_manager;
-  GdkDevice        *device;
-  GdkCursor        *cross;
-  GdkWindow        *root;
-  GList            *devices, *l;
+  WnckHandle *handle;
+  GdkWindow *root;
+  GdkDisplay *display;
+  GdkSeat *seat;
+  GdkCursor *cross;
+  GdkSeatCapabilities caps;
+  GdkGrabStatus status;
+
+  handle = data;
 
-  dev_manager = gdk_display_get_device_manager (gdk_display_get_default ());
   root = gdk_get_default_root_window ();
+  display = gdk_display_get_default ();
+  seat = gdk_display_get_default_seat (display);
+  cross = gdk_cursor_new_for_display (display, GDK_CROSS);
+  caps = GDK_SEAT_CAPABILITY_POINTER | GDK_SEAT_CAPABILITY_KEYBOARD;
 
-  gdk_window_add_filter (root, (GdkFilterFunc) target_filter, NULL);
+  gdk_window_add_filter (root, (GdkFilterFunc) target_filter, handle);
 
-  cross = gdk_cursor_new (GDK_CROSS);
-  device = gdk_device_manager_get_client_pointer (dev_manager);
-  status = gdk_device_grab (device, root, GDK_OWNERSHIP_WINDOW, TRUE,
-                            GDK_BUTTON_PRESS_MASK, cross, GDK_CURRENT_TIME);
+  status = gdk_seat_grab (seat, root, caps, TRUE, cross, NULL, prepare, NULL);
   g_object_unref (cross);
 
   if (status != GDK_GRAB_SUCCESS)
     {
-      g_warning ("Pointer grab failed.\n");
-      clean_up ();
+      g_warning ("Seat grab failed.");
+      clean_up (handle);
       return FALSE;
     }
 
-  devices = gdk_device_manager_list_devices (dev_manager, GDK_DEVICE_TYPE_MASTER);
-
-  for (l = devices; l; l = l->next)
-    {
-      device = GDK_DEVICE (l->data);
-
-      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
-        continue;
-
-      status = gdk_device_grab (device, root, GDK_OWNERSHIP_NONE, TRUE,
-                                GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK, NULL,
-                                GDK_CURRENT_TIME);
-
-      if (status != GDK_GRAB_SUCCESS)
-        break;
-    }
-
-  g_list_free (devices);
-
-  if (status != GDK_GRAB_SUCCESS)
-    {
-      g_warning ("Keyboard grab failed.\n");
-      clean_up ();
-      return FALSE;
-    }
-
-  gdk_flush ();
+  gdk_display_flush (display);
 
   return FALSE;
 }
 
 static void
-clean_up (void)
+clean_up (WnckHandle *handle)
 {
   GdkWindow *root;
-  GdkDeviceManager *dev_manager;
-  GdkDevice        *device;
-  GList            *devices, *l;
+  GdkDisplay *display;
+  GdkSeat *seat;
 
   root = gdk_get_default_root_window ();
-  dev_manager = gdk_display_get_device_manager (gdk_display_get_default ());
-  gdk_window_remove_filter (root, (GdkFilterFunc) target_filter, NULL);
+  display = gdk_display_get_default ();
+  seat = gdk_display_get_default_seat (display);
 
-  device = gdk_device_manager_get_client_pointer (dev_manager);
-  gdk_device_ungrab (device, GDK_CURRENT_TIME);
-
-  devices = gdk_device_manager_list_devices (dev_manager, GDK_DEVICE_TYPE_MASTER);
-
-  for (l = devices; l; l = l->next)
-    {
-      device = GDK_DEVICE (l->data);
-
-      if (gdk_device_get_source (device) != GDK_SOURCE_KEYBOARD)
-        continue;
-
-      gdk_device_ungrab (device, GDK_CURRENT_TIME);
-    }
-
-  g_list_free (devices);
+  gdk_window_remove_filter (root, (GdkFilterFunc) target_filter, handle);
+  gdk_seat_ungrab (seat);
 
   gtk_main_quit ();
 }
@@ -1886,12 +1868,15 @@ main (int argc, char **argv)
   GOptionContext *ctxt;
   GOptionGroup   *group;
   GError         *error;
+  WnckHandle     *handle;
   WnckScreen     *screen;
 
   bindtextdomain (GETTEXT_PACKAGE, WNCK_LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
+  gdk_set_allowed_backends ("x11");
+
   ctxt = g_option_context_new (NULL);
   g_option_context_set_translation_domain (ctxt, GETTEXT_PACKAGE);
 
@@ -1956,17 +1941,19 @@ main (int argc, char **argv)
 
   gtk_init (&argc, &argv);
 
-  wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
+  handle = wnck_handle_new (WNCK_CLIENT_TYPE_PAGER);
 
   if ((option_screen && interact_screen < 0) || !option_screen)
-    screen = wnck_screen_get_default ();
+    screen = wnck_handle_get_default_screen (handle);
   else
     {
-      screen = wnck_screen_get (interact_screen);
+      screen = wnck_handle_get_screen (handle, interact_screen);
       if (!screen)
         {
-         g_printerr (_("Cannot interact with screen %d: "
-                       "the screen does not exist\n"), interact_screen);
+          g_printerr (_("Cannot interact with screen %d: "
+                        "the screen does not exist\n"), interact_screen);
+
+          g_object_unref (handle);
           return 0;
         }
     }
@@ -1986,12 +1973,15 @@ main (int argc, char **argv)
 
   if (get_from_user)
     {
-      g_idle_add (get_target, NULL);
+      g_idle_add (get_target, handle);
 
       gtk_main ();
 
       if (!got_from_user)
-        return 0;
+        {
+          g_object_unref (handle);
+          return 0;
+        }
     }
 
   if (mode == SCREEN_READ_MODE)
@@ -2031,7 +2021,7 @@ main (int argc, char **argv)
       if (got_from_user)
         class_group = wnck_window_get_class_group (got_from_user);
       else
-        class_group = wnck_class_group_get (interact_class_group);
+        class_group = wnck_handle_get_class_group (handle, interact_class_group);
 
       if (class_group)
         {
@@ -2056,7 +2046,7 @@ main (int argc, char **argv)
       if (got_from_user)
         app = wnck_window_get_application (got_from_user);
       else
-        app = wnck_application_get (interact_app_xid);
+        app = wnck_handle_get_application (handle, interact_app_xid);
 
       if (app)
         {
@@ -2079,7 +2069,7 @@ main (int argc, char **argv)
       if (got_from_user)
         window = got_from_user;
       else
-        window = wnck_window_get (xid);
+        window = wnck_handle_get_window (handle, xid);
 
       if (window)
         {
@@ -2095,5 +2085,7 @@ main (int argc, char **argv)
                       "the window cannot be found\n"), xid);
     }
 
+  g_object_unref (handle);
+
   return 0;
 }
